@@ -8,72 +8,76 @@ import pyodbc
 import numpy as np
 import pandas as pd
 from itertools import chain
-from fracdiff import StationaryFracdiff
 from enigmx.utils import (
     open_zarr_general, 
     check_zarr_data,
     sel_days,
-    forwardFillOneDimension,
-    vectorizedSimpleVwap,
-    volumeBarConstruction,
-    tickBarConstruction,
     __volatility__,
     get_horizons,
-    __vectorizedVwap__,
-    __tickBarConstruction__,
-    __volumeBarConstruction__,
-    simpleBarVolume,
-    simpleBarTick,
-    imb_feat,
     barrierCoords,
-    iterativeVwap,
     __newTickBarConstruction__,
     __newVolumeBarConstruction__,
     __newDollarBarConstruction__,
     infoBarGenerator,
-    barsNameDefinition
     )
 
 #1. Main Quantmoon SQL Manager
 class QuantmoonSQLManager(object):
+    """
+    Clase SQL Manager. 
     
+    Instancia general del Data Bundle.
+    
+    Métodos principales:
+        - 'create_new_database'
+        - 'select_database'
+        - 'create_new_table'
+        - 'read_table_info'
+        - 'get_write_statement'
+        - 'get_write_statement'
+        - 'write_table_info'
+        - 'globalSimpleInsert'
+        
+    Métodos interiores:
+        - '__checkDatabase__'
+        - '__getTables__'
+        - '__checkTable__'
+        - '__configurateTableStatement__'
+        - '__configurateWrittingInfo__'
+        
+    """
     def __init__(self, server, database_name, 
-                 base_database='TSQL', 
-                 vwap=False, globalRange=False, 
-                 remote=False, loggin='', access=''):
+                 base_database='TSQL', globalRange=False,
+                 loggin='', access=''):
         
         self.server = server
         self.base_database = base_database
-        self.vwap = vwap
+
         self.globalRange = globalRange
-        self.remote = remote
         
         self.loggin = ('Driver={SQL Server};Server='+server+
                        ';Trusted_Connection=yes;')
         
-        if self.vwap:
-            self.database_name = database_name.upper() + '_VWAP'
-        else:
-            self.database_name = database_name.upper() + '_NOVWAP'
+        self.database_name = database_name.upper() 
             
         if self.globalRange:
             self.global_ = '_GLOBAL'
         else:
             self.global_ = '_NOGLOBAL'
             
-            
         self.access = ('Driver={SQL Server};Server='+ server +
                        ';Database='+ self.database_name +
                        ';Trusted_Connection=yes;')
         
     def __checkDatabase__(self):
+        # revisar si la base de datos ya existe con ese nombre
         if self.database_name.lower() == self.base_database.lower():
             raise ValueError(
                 "Database name's equivalent already exist. Change it."
                 )
             
     def __getTables__(self, cursor): 
-        
+        # obtener las tablas seleccionadas
         statement = 'SELECT name FROM {}.sys.Tables'.format(
                                             self.database_name
                                             )        
@@ -87,7 +91,7 @@ class QuantmoonSQLManager(object):
             return [db[0] for db in cursor] 
         
     def __checkTable__(self, cursor, table_name):
-
+        # revisar si ya existen las tablas 
         self.tableName = table_name + self.global_
         return self.tableName in self.__getTables__(cursor)
     
@@ -95,44 +99,59 @@ class QuantmoonSQLManager(object):
                                       table_name, 
                                       dictColVal):
         
+        # configurar la tabla
         __base_statement__ = ('''CREATE TABLE {} ({})''')
         
+        # generar sentencia para la config. de la tabla
         sentence_ = ('{} {},' * len(dictColVal))[:-1]
         table_elements = tuple(chain(*dictColVal.items()))
         tablestructure_ = sentence_.format(*table_elements)
         
+        # construcción de nombre de la tabla
         self.tableName = table_name + self.global_
         
+        # genera el statement para escritura de la tabla
         return __base_statement__.format(self.tableName, tablestructure_)
     
     def __configurateWrittingInfo__(self, database_name, 
                                     table_name, cursor_):
         
+        # configurar nombre de la tabla para conf. escritura
         self.tableName = table_name + self.global_
         
+        # realiza búsqueda de columnas según la tabla
         searchColumnsName = (
             "SELECT COLUMN_NAME FROM {}.INFORMATION_SCHEMA.COLUMNS " 
             "WHERE TABLE_NAME = N'{}'").format(
-                            self.database_name, self.tableName
+                            database_name, self.tableName
                             )
-
                 
+        # ejecuta el cursor con la búsqueda de columnas
         cursor_.execute(searchColumnsName)
+        
+        # extrae los nombres de las columnas
         SQLCOLNAMES = [element[0] for element in cursor_]
+        
+        # verifica la cant de columnas
         SQLCOLNUM = len(SQLCOLNAMES)
         
+        # añade un '?' para agregado de elementos x columna
         base_values = ("?," * (SQLCOLNUM))[:-1]
         
+        # define INSERT statement
         insert_ = "INSERT INTO {}.dbo.{}".format(
                                 self.database_name, 
                                 self.tableName
                                 )
         
+        # construye predicado de inserción
         _predicative_ = ("{}," * SQLCOLNUM)[:-1]
         predicate_ = _predicative_.format(*SQLCOLNAMES)
         
+        # añade los valores a insertar
         values_ = "VALUES ({})".format(base_values)
         
+        # define el statement general para usarse en SQL
         final_statement = "{} ({}) {}".format(
                                 insert_, 
                                 predicate_, 
@@ -142,15 +161,23 @@ class QuantmoonSQLManager(object):
             
     def create_new_database(self):
         
+        # método para crear una nueva base de datos
+        
+        # chequear que no exista previamente dicha base de datos
         self.__checkDatabase__()
         
+        # loggearse a la base de datos
         dbconn = pyodbc.connect(self.loggin)
+        
+        # definición y ejecución del cursor
         cursor = dbconn.cursor()
         cursor.execute("SELECT name FROM master.dbo.sysdatabases")
         existence_ = [db[0].lower() for db in cursor]
         
+        # extracción del nombre de la base de datos
         data_base_name = self.database_name
         
+        # si la base de datos no existe, créala
         if data_base_name not in existence_:
             dbconn.autocommit = True
             cursor.execute("CREATE DATABASE [{}]".format(
@@ -160,9 +187,14 @@ class QuantmoonSQLManager(object):
             print("Database '%s' successfully created" % 
                                           self.database_name
                                           )
+            # cierra el cursor y la conexión a la base de datos
             cursor.close()
             dbconn.close()
+            
+        # en caso exista la base de datos, interrumpe el proceso
         else:
+            
+            # cierra el cursor y la conexión a SQL
             cursor.close()
             dbconn.close()
             raise ValueError(
@@ -170,13 +202,21 @@ class QuantmoonSQLManager(object):
                 )
             
     def select_database(self):
+        # método para seleccionar una base de datos
         
+        # intenta
         try: 
+            # conexión a la base de datos y creación de cursor
             dbconn = pyodbc.connect(self.access)
             cursor = dbconn.cursor()
+            
+        # en caso surga un error:
         except pyodbc.Error as ex:
+            # imprime el error de conexión fallida
             print("Error statement appears:{}".format(ex))
             return "Circuit Breaker: Non SQL Connection initialized"
+        
+        # si todo marcha bien, retorna conexión base de datos y cursor
         return dbconn, cursor 
         
     def create_new_table(self, 
@@ -184,21 +224,27 @@ class QuantmoonSQLManager(object):
                          dict_info, 
                          dbconn_, 
                          cursor_):
-
+        
+        # método para crear una nueva tabla
+        
+        # chequea primero si la tabla no existe
         if self.__checkTable__(cursor_, table_name):
+            # si ya existe con ese nombre, retorna error
             raise ValueError(
                     "Table's name already exists. "
                     "Change it or use 'write_table_info()' "
                     "method instead."
                 )
-        
+        # caso contrario, genera el statement para crear la tabla
         statement = self.__configurateTableStatement__(
                                         table_name, dict_info
                                         )
-        
+        # intenta la creación de tabla
         try:
             cursor_.execute(statement)
             dbconn_.commit()
+            
+        # en caso error, muéstralo
         except pyodbc.Error as ex:
             print("Error statement computation: {}".format(ex))
             return "Breaks: Creating Table Process Failed"
@@ -208,17 +254,29 @@ class QuantmoonSQLManager(object):
                 )
         
     def read_table_info(self, statement, dbconn_, cursor_, dataframe=False):
+        # método para lectura de información de tabla
         
+        # si no se quiere un dataframe como resultado
         if dataframe==False:
+            # ejecuta la lectura del statement
             cursor_.execute(statement)     
+            
+            # retorna el cursor
             return cursor_
+        
+        #si se quiere un dataframe
         else:
+            # lectura de la tabla
             sql_query = pd.read_sql_query(statement, dbconn_)
+            
+            # retorna la tabla
             return sql_query 
     
     def get_write_statement(self, database_name, 
                             table_name, cursor_):
-        
+        """
+        Genera el statement (str) de escritura para tabla SQL.
+        """
         return self.__configurateWrittingInfo__(
                                         database_name, 
                                         table_name, 
@@ -227,40 +285,157 @@ class QuantmoonSQLManager(object):
         
     def write_table_info(self, statement_, 
                          dbconn_, cursor_, 
-                         data_, bartype_,
-                         vwap_):
+                         data_, bartype_):
+        # método para escribir información en las tablas
         
+        # dado un df, itera por cada fila y llénalo en la tabla selecccionada
         for idx, row in data_.iterrows():
             
-            if bartype_=='time' and vwap_:
-                cursor_.execute(
-                    statement_, row.datetime, row.price
+            # si el tipo de tabla es TUNNING 
+            if bartype_.upper() == 'TUNNING':
+                cursor_.execute( 
+                    statement_, 
+                    row.tick_t, 
+                    row.volume_t, 
+                    row.dollar_t, 
+                    row.stock
                     )
-            elif bartype_=='time' and vwap_==False:
+                
+            # si el tipo de tabla es para las barras o barras sampleadas
+            elif bartype_.upper()=='BARS' or  bartype_.upper()=='BARS_SAMPLED':
                 cursor_.execute(
-                    statement_, row.datetime, row.price, row.vol
-                    )     
-            elif bartype_=='tick' and vwap_:
-                cursor_.execute(
-                    statement_, row.grpId, row.datetime, row.price
+                    statement_, 
+                    row.open_price, 
+                    row.high_price, 
+                    row.low_price, 
+                    row.close_price, 
+                    row.open_date, 
+                    row.high_date, 
+                    row.low_date, 
+                    row.close_date, 
+                    row.basic_volatility, 
+                    row.bar_cum_volume, 
+                    row.vwap, 
+                    row.fracdiff,
+                    row.volatility, 
+                    row.horizon,
+                    row.upper_barrier,
+                    row.lower_barrier                    
                     )
-            elif bartype_=='tick' and vwap_==False:
+            
+            # si el tipo de tabla es para el cómputo de la entropía
+            elif bartype_.upper() == 'ENTROPY':
                 cursor_.execute(
-                    statement_, row.datetime, row.price
-                    )                
-            elif bartype_=='fracdiff':
+                    statement_, 
+                    row.open_price, 
+                    row.high_price, 
+                    row.low_price, 
+                    row.close_price, 
+                    row.open_date, 
+                    row.high_date, 
+                    row.low_date, 
+                    row.close_date, 
+                    row.basic_volatility, 
+                    row.bar_cum_volume, 
+                    row.vwap, 
+                    row.fracdiff,
+                    row.volatility, 
+                    row.horizon,
+                    row.upper_barrier,
+                    row.lower_barrier,
+                    row.entropy                    
+                    )            
+                
+            # si el tipo de tabla es para el cómputo del etf trick
+            elif bartype_.upper() == 'ETFTRICXCK':
                 cursor_.execute(
-                    statement_, row.datetime, row.fracdiff
+                    statement_, 
+                    row.value, 
+                    row.high_price, 
+                    row.low_price, 
+                    row.close_price, 
+                    row.open_date, 
+                    row.high_date, 
+                    row.low_date, 
+                    row.close_date, 
+                    row.basic_volatility, 
+                    row.bar_cum_volume, 
+                    row.vwap, 
+                    row.fracdiff,
+                    row.volatility, 
+                    row.horizon,
+                    row.upper_barrier,
+                    row.lower_barrier,
+                    row.entropy                    
+                    )               
+                
+            # si el tipo de tabla es para el cómputo de las barras completas (no finales)
+            elif bartype_.upper() == 'BARS_COMPLETED':
+                cursor_.execute(
+                    statement_, 
+                    row.open_price, 
+                    row.high_price, 
+                    row.low_price, 
+                    row.close_price, 
+                    row.open_date, 
+                    row.high_date, 
+                    row.low_date, 
+                    row.close_date, 
+                    row.basic_volatility, 
+                    row.bar_cum_volume, 
+                    row.vwap, 
+                    row.fracdiff,
+                    row.volatility, 
+                    row.horizon,
+                    row.upper_barrier,
+                    row.lower_barrier,
+                    row.barrierPrice,
+                    row.barrierLabel,
+                    row.barrierTime
+                    )            
+                
+            # si el tipo de tabla es para el cómputo de las barras con weight y label (final)
+            elif bartype_.upper() == 'BARS_WEIGHTED':
+                cursor_.execute(
+                    statement_, 
+                    row.open_price, 
+                    row.high_price, 
+                    row.low_price, 
+                    row.close_price, 
+                    row.open_date, 
+                    row.high_date, 
+                    row.low_date, 
+                    row.close_date, 
+                    row.basic_volatility, 
+                    row.bar_cum_volume, 
+                    row.vwap, 
+                    row.fracdiff,
+                    row.volatility, 
+                    row.horizon,
+                    row.upper_barrier,
+                    row.lower_barrier,
+                    row.barrierPrice,
+                    row.barrierLabel,
+                    row.barrierTime,
+                    row.overlap,
+                    row.weight,
+                    row.weightTime
+                    )                         
+            
+                
+            else: 
+                raise ValueError(
+                    "Not recognized database name. \
+                        Normally bartype's name or 'TUNNING'. Please check."
                     )
+                    
         dbconn_.commit()
-        #cursor_.close()
-        print("Information '{}' was written".format(
-                                        self.database_name
-                                        )
-                                    )
+
+        print("---Information was written---")
         
     def globalSimpleInsert(self, statement_, infoTuple, dbconn_, cursor_):
         
+        # método simplificado para inserción de elementos genéricos en tabla
         for idx in range(0, infoTuple[0].shape[0]):
             cursor_.execute(
                 statement_, 
@@ -279,7 +454,22 @@ class QuantmoonSQLManager(object):
 
 #2. Data Repository Initialization | General Class for Feature Construction                
 class DataRespositoryInitialization(object):
+    """
+    Clase para el manejo del repositorio de datos Zarr a SQL. 
     
+    Clase DataRepositoryInitialization:
+        
+    
+    Métodos principales:
+        - 'bar_novect_construction': permite construcción no vectorizada de las barras
+        - 'generateTripleBarrier': DEPRECATED 
+        - 'geneticIterativeFunction': DEPRECATED
+        - 'general_matrix': DEPRECATED
+
+    Métodos interiores:
+        - '__makeTupleContent__'
+
+    """
     def __init__(self, 
                  data_dir, 
                  stock, 
@@ -454,7 +644,6 @@ class DataRespositoryInitialization(object):
         if bartype=='tick':
             
             #get list of column names for dataframe construction 
-            #colNames = barsNameDefinition(bartype)
             
             #results_ tuple info (last arg. is 'alpha_calibration')   
             result_info = __newTickBarConstruction__(
@@ -477,26 +666,10 @@ class DataRespositoryInitialization(object):
                     ]
                 )
             
-            
-            
-            #print(result_dataset, type(result_dataset))            
-            #print(caca)
-            
-            #dataframe computation | first only OHLC info
-            #result_dataset = pd.DataFrame.from_records(
-            #    resultInfo[0], 
-            #    columns=colNames
-            #    )
-            
-
-            
-            #adding vwap bar information 
-            #result_dataset[bartype + "_vwap"] = resultInfo[1]
                   
         elif bartype=='volume':
             
             #get list of column names for dataframe construction 
-            #colNames = barsNameDefinition(bartype)
             
             #results_ tuple info (last arg. is 'alpha_calibration')   
             result_info = __newVolumeBarConstruction__(
@@ -518,20 +691,10 @@ class DataRespositoryInitialization(object):
                     np.array(resultInfo[0]), resultInfo[1]
                     ]
                 )
-                                    
-            #dataframe computation | first only OHLC info
-            #result_dataset = pd.DataFrame.from_records(
-            #    resultInfo[0], 
-            #    columns=colNames
-            #    )
-            
-            #adding vwap bar information  
-            #result_dataset[bartype + "_vwap"] = resultInfo[1]
             
             
         elif bartype=='dollar':
             #get list of column names for dataframe construction 
-            #colNames = barsNameDefinition(bartype)
             
             #results_ tuple info (last arg. is 'alpha_calibration')   
             result_info = __newDollarBarConstruction__(
@@ -552,30 +715,15 @@ class DataRespositoryInitialization(object):
                     np.array(resultInfo[0]), resultInfo[1]
                     ]
                 )
-                        
-            
-            #dataframe computation | first only OHLC info
-            #result_dataset = pd.DataFrame.from_records(
-            #    resultInfo[0], 
-            #    columns=colNames
-            #    )
-            
-            #adding vwap bar information  
-            #result_dataset[bartype + "_vwap"] = resultInfo[1]            
+
       
-        #PENDIENTE!!!!!!!    
+        # ---------------- pendiente ------------------------
         elif bartype == 'imbalance':
             raise ValueError(
                 "Bartype 'imbalance' not currently available.\
                 Check 'bar_novect_construction-databundle.py'."
                 )
             
-            #result_dataset = imb_feat(
-            #                    info_tuple[1][0], 
-            #                    info_tuple[2][0], 
-            #                    imbalance_list
-            #                )
-            
         return result_dataset 
 
     def generateTripleBarrier(self, 
@@ -583,11 +731,13 @@ class DataRespositoryInitialization(object):
                               window_volatility=1, 
                               window_horizon=1, 
                               tabular=False):
-    
+        """
+        Deprecated Method to compute Triple Barrier.
+        """
+        
+        # método para el cálculo de la triple barrera 
         
         #openPrice at each datapoint | price bars
-        #precio open de cada día | "result_dataset = info_tuple[1][0][0]" 
-        #no útil porque se tiene el precio open/close de cada barra
         df_, concated_volbar = (
             dictInfo['special_time'], 
             dictInfo["volume"]
@@ -626,8 +776,6 @@ class DataRespositoryInitialization(object):
             axis=1
         ).dropna()
         
-        #ya ejecuta la apertura del zarr
-        
         tripleBarrier = barrierCoords(
             path = self.data_dir+"/"+ self.stock +".zarr", 
             initHorizon = df_.index.values,
@@ -680,6 +828,12 @@ class DataRespositoryInitialization(object):
                                  window_fracdiff = 2,
                                  daily_time_bars_organization = None):
         
+        """
+        Deprecated method to organize an iterative bar-no-vect-construction.
+        """
+        
+        # método en desuso para computar las barras 
+        
         info_tuple = self.__makeTupleContent__(freq, time)
         
         daily_time_bars = info_tuple[3]
@@ -706,6 +860,11 @@ class DataRespositoryInitialization(object):
                        window_fracdiff=2,
                        tabular_output=False):
         """
+        
+        DEPRECATED METHOD FOR GENERAL MATRIX CONSTRUCTION
+        
+        Método para construir una matriz general de eventos x día.
+        
         dictInfo results:
             - Fracdiff: daily Fracdiff based on VWAP each day
             - tick: total tick bars in a specific day
@@ -726,9 +885,8 @@ class DataRespositoryInitialization(object):
                 y luego modificar esta parte.
         
         """
-        #important: 
-        #'parse_dates = [COLUMN OF DATES]' 
-        #in 'read_csv' allows to transform str dates to datetime        
+        
+        # método depreciado para construcción de eventos x día inc. barras
         dictInfo = dataframes_dict
         
         resultBarrier = self.generateTripleBarrier(
