@@ -459,7 +459,7 @@ def iterativeVwap(list_arrayPrice, list_arrayVol):
     result = []
         
     for idx in range(len(list_arrayPrice)):
-        
+
         if np.sum(list_arrayVol[idx]) == 0:
             vwap_ = np.sum(
                 list_arrayPrice[idx]*list_arrayVol[idx]
@@ -711,97 +711,6 @@ def get_prod(arr1,arr2):
 #######----------------------------------------------------------------####### 
 ####### Main IMBALANCE BARS Function #######
 
-#Hypertunning Variables per stock for Imbalance Bars Construction
-def comparative_df(
-        list_stocks, 
-        dates, 
-        alpha_1_interval, 
-        alpha_2_interval, 
-        bars_interval,
-        path, hyperp_path, 
-        tipo = 'DIB',
-        drop_dup = False, plot = False):
-    
-    """
-    Main Function to define Imbalance Bars parameters.
-    
-    It saves paratmers for each stock like:
-                            A            AA      ...
-        alpha_1      9.00E-05      9.00E-05      ...
-        alpha_2      1.00E-05      5.00E-05      ...    
-        num_bars           12            14      ...    
-        ET_init   1215.147727   1578.525974      ...    
-        Eb_init  -0.013919436  -0.010053447      ...    
-        Ebv_init -221.2989616  -88.02652877      ...    
-        
-    This function returns a .csv in certain path to use during Imbalance 
-    construction.
-    """
-    
-    df = pd.DataFrame(
-        columns=[
-            'symbol', 'num_bars','alpha1',
-            'alpha2','len_Ts','max_threshold',
-            'min_threshold'
-            ]
-        )
-    
-    hyperp = {}
-    
-    for symbol in list_stocks:
-        print(symbol)
-        ts_ref = 1
-        val=[0.0001,0.0001,5,1,1,1]
-        df_init = load_data(
-            symbol, path, dates, drop_dup = True
-            )
-        
-        for num_bars in bars_interval:
-            X, ET_init, Eb_init, Ebv_init = init_values(
-                df_init, 
-                tipo=tipo, 
-                num_bars=num_bars
-                )
-            X = np.array(X.bv,dtype=np.float64)
-            
-            for alpha_1 in alpha_1_interval:
-                
-                for alpha_2 in alpha_2_interval:
-                    Ts, thres = compute_Ts_DIB(
-                            X, 
-                            ET_init, 
-                            Ebv_init, 
-                            alpha_1, 
-                            alpha_2, 
-                            50, 
-                            len(dates)
-                        )
-                    
-                    if len(Ts)>ts_ref:
-                        
-                        if thres[-1]<thres[-2]*10:
-                            val[0] = alpha_1
-                            val[1] = alpha_2
-                            val[2] = num_bars
-                            val[3] = ET_init
-                            val[4] = Eb_init
-                            val[5] = Ebv_init
-                            ts_ref = len(Ts)
-                            
-        hyperp[symbol] = val
-        
-    df = pd.DataFrame(
-        hyperp,index = [
-            'alpha_1','alpha_2',
-            'num_bars',
-            'ET_init',
-            'Eb_init','Ebv_init'
-            ]
-        )
-    df.to_csv(hyperp_path+"hyperp.csv")
-                
-    return "Imbalance Hyperparameters Tunned"
-    
 def input_(df, init_b ,tipo):
     '''
     init_b: valor inicial del tick direction, puede ser 1 o -1, no deberia tener gran impacto
@@ -1383,30 +1292,88 @@ def simpleBarTick(arrayTime,
 def global_list_stocks(data_dir, 
                        common_path = '.zarr', 
                        drop_extension = 5):
+    
+    """
+    Función para obtener todas la lista de acciones de un repositorio local.
+    
+    Lee los nombres de los archivos existentes que tienen un 
+    string en común (prefijo, infijo, sufijo).
+    
+    Elimina los strings innecesarios y se queda con el nombre de las acciones.
+    
+    Inputs:
+        - data_dir (str): local path para buscar.
+        - common_path (str): tipo de su-pre-in fijo para buscar.
+        - drop_extension (int): # de eliminación de los últimos #
+        
+    outputs:
+        - lista de acciones (lista de strings)
+    """
+    
     list_stocks = []
     for i,file in enumerate(os.listdir(data_dir)):
         if file.endswith(common_path):
             list_stocks.append(os.path.basename(file)[:-drop_extension])
     return list_stocks
 
-#Data Tunning Preparation
+# nominación de lista de acciones para ingesta directa de path 
+EquitiesEnigmxUniverse = global_list_stocks('D:/data_repository/')
+
+#Data Tunning Preparation| CAMBIARLO POR SQL 
 def dataPreparation_forTuning(csv_path,
-                              list_features_names, 
-                              label_name,
+                              label_name = "barrierLabel",
+                              features_sufix = "feature",
+                              timeIndexName = "close_date",
                               set_datetime_as_index = True):
+    """
+    Función para preparar un csv para la utilización con los modelos.
+    
+    Inputs:
+        - csv_path (str): dirección local donde se encuentra el csv
+        - label_name (str): nombre del label en dicho pandas extraído del csv-path.
+        - features_sufix (str): sufijo de las columnas de features en el df.
+        - timeIndexName (str): nombre de la columna temporal a usarse como Idx de evento.
+        - set_datetime_as_index (bool): setear el idx de evento como idx del dataframe.
+        
+    Output:
+        - Tuple:
+            * X : matriz de features
+            * y : vector de label
+            * t1: vector de timeIndex 
+    """
+    
     if not set_datetime_as_index:
         raise ValueError(
             "Only True available for datatime as index until now."
             )
-        
-    df = pd.read_csv(
-        csv_path, index_col='datetime'
+    
+    # obtención del df stacked desde path
+    dfStacked = pd.read_csv(
+        csv_path, 
         ).dropna()
     
-    X = df[list_features_names]
+    # seteo de la columna timeIndexName como un datetime (no como string)
+    dfStacked[timeIndexName] = dfStacked[timeIndexName].astype('datetime64[ns]')
     
-    y = df[label_name]
+    # definir cono index a la columna asignada a timeIndexName
+    dfStacked = dfStacked.set_index(timeIndexName)
     
+    # array conteniendo los nombres de features según sufijo definido
+    featureColumnsName = dfStacked.filter(
+                like= features_sufix
+                ).columns.values   
+    
+    # selección únicamente de features
+    X = dfStacked[featureColumnsName]
+    
+    # selección únicamente de label
+    y = dfStacked[label_name]
+    
+    # ordenamiento serie-temporal según timeIndexName
+    X = X.sort_index()
+    y = y.sort_index()
+    
+    # obtención de vector de timeIndex
     t1 = pd.Series(data=y.index, index=y.index)    
     
     return X, y, t1
@@ -1427,7 +1394,53 @@ keep_same = {
     'timeAchieved', 'time','special_time'
 }
 
-def dataPreparation(data, 
+def dataPreparation(data_csv, 
+                    feature_sufix = 'feature',
+                    label_name='barrierLabel', 
+                    timeIndexName = "close_date",
+                    timeLabelName = "horizon"): 
+    
+    """
+    Función para preparar un csv para la utilización con los modelos.
+    
+    Inputs:
+        - csv_path (str): dirección local donde se encuentra el csv
+        - label_name (str): nombre del label en dicho pandas extraído del csv-path.
+        - features_sufix (str): sufijo de las columnas de features en el df.
+        - timeIndexName (str): nombre de la columna temporal a usarse como Idx de evento.
+        - set_datetime_as_index (bool): setear el idx de evento como idx del dataframe.
+        
+    Output:
+        - Tuple:
+            * X : matriz de features
+            * y : vector de label
+            * t1: vector de timeIndex 
+    """    
+    
+    dfStacked = pd.read_csv(
+        data_csv, 
+        ).dropna()
+    
+    dfStacked[timeIndexName] = dfStacked[timeIndexName].astype('datetime64[ns]')
+    dfStacked[timeLabelName] = dfStacked[timeLabelName].astype('datetime64[ns]')
+    
+    dfStacked = dfStacked.set_index(timeIndexName)
+    
+    featureColumnsName = list(dfStacked.filter(
+                like= feature_sufix
+                ).columns.values)       
+    
+    featureColumnsName.append(timeLabelName)
+    
+    X = dfStacked[featureColumnsName]
+
+    y = dfStacked[label_name]
+    
+    X[label_name] = y
+     
+    return X
+
+def dataPreparationOld(data, 
                     label_name='tripleBarrier', 
                     add_suffix = True, 
                     no_suffix = keep_same):
@@ -1545,7 +1558,7 @@ def purge_embargo(X,train_indices,test_indices,embargo):
     train_2 = train[train.index > (end + np.timedelta64(embargo,'D'))]
     
     new_train = pd.concat([train_1,train_2])
-    train['new_index'] = train_indices.copy()
+    train.loc[:, 'new_index'] = train_indices.copy()
     train = pd.merge(train,new_train,
                      how='left',
                      left_index  = True, 
@@ -1763,6 +1776,7 @@ def __newTickBarConstruction__(arrayTime,
      
     return groupTime, groupPrice, groupVol
 
+#New Volume Bar Construction Function | No Vectorized
 def __newVolumeBarConstruction__(arrayTime, 
                                  arrayPrice, 
                                  arrayVol, 
@@ -1814,7 +1828,11 @@ def __newDollarBarConstruction__(arrayTime,
                                  arrayPrice, 
                                  arrayVol, 
                                  alpha_calibration=1e3):
-
+    """
+    New dollar bar construction vectorized base version.
+    
+    Deprecated
+    """
     #volume cumsum
     cumsumDol = np.cumsum(arrayPrice * arrayVol)
     
@@ -1865,6 +1883,7 @@ def OHLC_BAR(list_arrayPrice, list_arraytime, list_arrayVol):
     OHLC + VOLATILITY + VOLUME computation.
     
     Includes arrayTime values for OHLC prices.
+
     """
     barOHLC_plus_time_info = []
     
@@ -1950,6 +1969,9 @@ def infoBarGenerator(grp_time,
     and VWAP.
     """
     
+    grp_prices = List(grp_prices)
+    grp_vols = List(grp_vols)
+    
     if bartype == 'tick':
         #compute vwap from Price and Vol by bar
         vwap = iterativeVwap(grp_prices, grp_vols)     
@@ -1991,9 +2013,27 @@ def infoBarGenerator(grp_time,
     
     else:
         raise ValueError("Not recognized bartype string input")
+
+##############################################################################
+##############################################################################
+############### PRE-ELEMENTS FOR TRIPLE BARRIER COMPUTATION ##################
+##############################################################################
+##############################################################################
         
 def getTripleBarrierOHLC(df):
+    """
+    Función que permite computar los elementos básicos previos de una Triple Barrera.
     
+    Inputs:
+        - df (pd.DataFrame): pandas dataframe con valores OHLC de la barra.
+        
+    Output:
+        - dataframe con las sig. columnas nuevas:
+            * uppber_barrier
+            * lower_barrier
+            * barrier_price
+            * barrier_date
+    """
     #upper and loser barriers column definition
     df["upper_barrier"] = df.close * (1 + df.volatility)
     df["lower_barrier"] = df.close * (1 - df.volatility)
@@ -2144,10 +2184,10 @@ def barsNameDefinition(bartype,
     """
     return [bartype +  "_" + colName for colName in columnBaseNames]
     
-dataset_column_names = ["open", "high", "low", "close", "open_date", 
-                        "high_date", "low_date", "close_date", 
-                        "basic_volatility", "bar_cum_volume", 
-                        "vwap", "fracdiff"]
+dataset_column_names = ["open_price", "high_price", "low_price", "close_price", 
+                        "open_date", "high_date", "low_date", "close_date", 
+                        "basic_volatility", "bar_cum_volume", "vwap", 
+                        "fracdiff"]
 
 ##############################################################################
 ##############################################################################
@@ -2167,7 +2207,31 @@ def ErrorIndexIdxFirstTrue(param):
         #only 1st value from tuple of single array
         return param[0][0]
 
-def vectorizedTripleBarrier(path, init, init_ts, last, last_ts, upper_bound, lower_bound):
+def vectorizedTripleBarrier(path, init, init_ts, last, last_ts, upper_bound, 
+                            lower_bound):
+    """
+    Función Principal para el cómputo de la Triple Barrera.
+    
+    Función ingestada en 'LabelTripleBarrierComputation'.
+    
+    Esta a su vez es ingestada en 'new_triple_barrier_computation'
+    de tripleBarrier.py a través del multiprocesador 'ray'.
+    
+    Inputs:
+        - path (str): dirección local donde se encuentran el .zarr de una acción det.
+        - init (datetime series): pd.Series conteniendo las fechas de c/ bar-event (close_date)
+        - init_ts (timestamp series): pd.Series conteniendo las fechas de c/ bar-event en formato timestamp
+        - last (datetime series): pd.Series conteniendo las fechas máx. de c/ label (horizon).
+        - last_ts (timestamp series): pd.Series conteniendo las fechas máx. de c/ label (horizon) en formato timestamp.
+        - upper_bound (float series): pd.Series conteniendo el valor float-price máximo de la barrera horizontal.
+        - lower_bound (float series): pd.Series conteniendo el valor float-price mínimo de la barrera horizontal.
+        
+    Output:
+        - Tupla conteniendo tres valores:
+            * finalPrice (float)
+            * finalLabel (int: -1, 0 ó 1)
+            * finalTimestamp (float)
+    """
     
     #days range to search tripleBarrier | timeframe values
     daysList = sel_days(init, last) 
@@ -2264,9 +2328,24 @@ def vectorizedTripleBarrier(path, init, init_ts, last, last_ts, upper_bound, low
 
 
 def LabelTripleBarrierComputation(barDataframe, stock, data_dir):
-
+    """
+    Función de ingesta e inicialización de la triple barrera.
+    
+    Ingesta la función "vectorizedTripleBarrier" para obtener resultados.
+    
+    Inputs:
+        - barDataframe (pd.DataFrame): dataframe central sobre el que computar la triple barrera.
+        - stock (str): nombre de la acción.
+        - data_dir (str): path donde se encuentra los archivos .zarr con data x tick.
+        
+    Output:
+        - barDataframe reformado inc. las siguientes columnas de la triple barrera:
+            * 'barrierPrice'
+            * 'barrierLabel'
+            * 'barrierTime'
+    """
     #tripleBarrier vectorized computation version
-    tripleBarrierInfo = barDataframe.apply( #para ENTROPY, aplicar lambda también como org. global
+    tripleBarrierInfo = barDataframe.apply( 
         lambda row: 
             vectorizedTripleBarrier(
                 path = data_dir + stock + ".zarr", 
@@ -2283,8 +2362,9 @@ def LabelTripleBarrierComputation(barDataframe, stock, data_dir):
             , 
             axis=1
             )
+
     #barrier columns information generation
-    barDataframe[['barrierPrice', 'barrierLabel', 'barrierTimestamp']] = \
+    barDataframe[['barrierPrice', 'barrierLabel', 'barrierTime']] = \
         pd.DataFrame(
             tripleBarrierInfo.tolist(), 
             index=tripleBarrierInfo.index
@@ -2334,13 +2414,10 @@ def saving_basic_bars(path_save, list_datasets, naming,
 
             saving_path = path_save + list_stocks[idx] + "_" + \
                 list_bar_names[idx_bartype].upper() + "_" + naming + '.csv'
-            
-            #save date as string to preserve mili/microsecond information
+
             dataframe.loc[:, 'open_date':'close_date'] = \
-                dataframe.loc[:, 'open_date':'close_date'].apply(
-                    lambda x: x.dt.strftime("%y-%m-%d %H:%M:%S.%f"),
-                    axis=1
-                    )
+                dataframe.loc[:, 'open_date':'close_date'].astype(str)
+
         
             dataframe.to_csv(
                 saving_path, 
@@ -2386,17 +2463,9 @@ def saving_etf_trick_or_sadf(path_save, list_datasets, naming,
 
         saving_path = path_save + "SERIES_" + \
                 bar_name.upper() + "_" + naming + '.csv'
-        
-        #save date as string to preserve mili/microsecond information
-        #dataset['time'] = \
-        #        dataset['time'].apply(
-        #            lambda x: x.dt.strftime("%y-%m-%d %H:%M:%S.%f"),
-        #            axis=1
-        #            )              
-               
+                
         dataset.to_csv(
                 saving_path, 
-                date_format="%y-%m-%d %H:%M:%S.%f", 
                 index=False
                 )    
         
@@ -2449,4 +2518,74 @@ def open_bar_files(base_path, stock, bartype):
     pandasBar = pd.read_csv(pandas_path, parse_dates=[
         "open_date","high_date","low_date","close_date", "horizon"]
                            )
+    #pandasBar = pd.read_csv(pandas_path)
+
     return pandasBar
+
+
+def construct_pandas_tunning(list_datasets, list_stocks):
+    """
+    Función que permite construir el pandas resumen del bars-tunning.
+    
+    Inputs:
+        - list_datasets (lst - df): lista de df conteniendo info tunning de cada acción
+        - list_stocks (lst - str): lista de strings con los nombres de las acciones
+    
+    Outputs:
+        - pandas Tunning conteniendo las sig. columnas
+            * [volume_t]
+            * [dollar_t]
+            * [stock]    
+    """
+    # lista de frames almacenable para concadenar
+    list_frames = []
+    
+    # iteración por idex y dataset
+    for idx, dataset in enumerate(list_datasets):
+        
+        # añade columna stock al dataset principal
+        dataset["stock"] = list_stocks[idx]
+        
+        # convierte al dataset en un pandas 
+        equity_tuning_frame_bar = pd.DataFrame(
+            dataset, 
+            index=[0]
+            )
+        
+        # añade a la lista final
+        list_frames.append(equity_tuning_frame_bar)
+    
+    # retorna tunning bars dataframe
+    return pd.concat(list_frames)
+
+
+##############################################################################
+
+def enigmxSplit(df, pct_average, backtest_comb = 0.5):
+    """
+    Función que permite el split aleatorio de un dataset según un % de partición.
+    
+    Es usado principalmente para dividir el stackedDf en tres elementos:
+        - stacked para FeatImportance
+        - stacked para ModelTunning
+        - stacked para Combinatorial backtest.
+    
+    Inputs:
+        - 'df' (pd.DataFrame)  : dataframe a particionar.
+        - 'pct_average' (float): valor ]0;1[ como % de primera partición.
+        - 'backtest_comb' (float): valor ]0;1[ como % de segunda partición.
+    """
+    # primera partición: obtener train y pre-test (div. según pct_average)
+    msk1 = np.random.rand(len(df)) < pct_average
+    
+    # obtención de primer train, y test
+    train, __test__ = df[msk1], df[~msk1]
+    
+    # segunda partición: obtener test y combinatorial data (div. 50% del resto)
+    msk2 = np.random.rand(len(__test__)) < backtest_comb 
+    
+    # obtención de test final y combinatorial dataset
+    test, combinatorial = __test__[msk2], __test__[~msk2]
+    
+    return train, test, combinatorial
+
